/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};

/*
*  Boilerplate code for creating computed data bindings
*/
document.addEventListener('gc-databind-ready', function () {
    gc.databind.registry.bind({
        size: 'widget.size.value',
        maxsize: 'widget.maxsize.value',
        failsafe: 'widget.failsafe.checked',
        secure: 'widget.secure.checked',
        isStatic: 'widget.isStatic.checked',
        fileToken: 'widget.fileToken.value',
        privateKey: 'widget.keyFilename.value',
        certificate: 'widget.certificate.value',
        signature: 'widget.signature.checked',
        readable: 'widget.readable.checked',
        writable: 'widget.writable.checked'
    }, 'widget.filetree.selectedFile', function (selectedFile) {
        return {
            size: selectedFile && selectedFile.fsize || 0,
            maxsize: selectedFile && selectedFile.maxsize || 0,
            failsafe: selectedFile && selectedFile.flagFailSafe || false,
            secure: selectedFile && selectedFile.flagSecure || false,
            isStatic: selectedFile && selectedFile.flagStatic || false,
            fileToken: selectedFile && selectedFile.fileToken || '',
            privateKey: selectedFile && selectedFile.key || '',
            certificate: selectedFile && selectedFile.certificateFileName || '',
            signature: selectedFile && selectedFile.flagNoSignatureTest || false,
            readable: selectedFile && selectedFile.flagPublicRead || false,
            writable: selectedFile && selectedFile.flagPublicWrite || false
        };
    });

    gc.databind.registry.bind('prop.activeFilePath', {
        activePage: 'widget.vtab.selectedLabel',
        activeFile: 'widget.filetree.selectedFile',
        isConnected: 'widget.statusBar.isHardwareConnected'
    }, function (values) {
        let isDir = false;
        let selectedFile;
        switch (values.activePage) {
            case 'User Files':
                selectedFile = values.activeFile;
                isDir = selectedFile && selectedFile.isDirectory;
                break;
        }

        let isDirty = false;

        // keep editor tab in sync with active editor
        if (selectedFile) {
            const editorTabs = document.querySelector('#editorTabs');
            const editorTab = editorTabs.findTextEditorTab(selectedFile.path);
            if (editorTab) {
                editorTabs.addTextEditorTab(selectedFile.name, selectedFile.path);
                isDirty = editorTab.dirty;
            } else {
                editorTabs.activeTabIndex = 0;
            }
        }

        if (values.isConnected) {
            gc.nav.enableAction('view.refresh');
            gc.nav.enableAction('file.add');
        } else {
            gc.nav.disableAction('view.refresh');
            gc.nav.disableAction('file.add');
        }

        if (selectedFile && isDirty && values.isConnected) {
            gc.nav.enableAction('file.save');
        } else {
            gc.nav.disableAction('file.save');
        }

        if (selectedFile && !selectedFile.isDirectory && values.isConnected) {
            gc.nav.enableAction('file.open');
            gc.nav.enableAction('file.delete');
            gc.nav.enableAction('file.download');
            gc.nav.enableAction('file.upload');

            return values.activeFile.path;
        } else {
            gc.nav.disableAction('file.open');
            gc.nav.disableAction('file.delete');
            gc.nav.disableAction('file.download');
            gc.nav.disableAction('file.upload');
            return '';
        }
    });

    gc.databind.registry.getBinding('prop.isCodecBusy').setValue(false);
    gc.databind.registry.bind('widget.busyIndication.visible', 'prop.isCodecBusy');

    const validateFilenames = /[\\\/*?|:"':]/;
    const validateFolders = /[\\*?|:"':]/;

    // Validation for New File Dialog
    gc.databind.registry.bind(
        {
            err: 'widget.dialog_submit.disabled',
            message: 'widget.dialog_error_message.label'
        }, {
            filename: 'widget.dialog_file_name.value',
            folder: 'widget.dialog_file_path.value',
            max: 'widget.dialog_max_size.value',
            size: 'prop.dialog_file_size',
            userFiles: 'uart.userFiles'
        }, function(values) {
            const file_matches = (values.filename || '').match(validateFilenames);
            const folder_matches = (values.folder || '').match(validateFolders);
            let message;
            if (file_matches) {
                message = `Invalid character ${file_matches[0]} in filename: ${values.filename}`;
            } else  if (folder_matches) {
                message = `Invalid character ${folder_matches[0]} in folder: ${values.folder}`;
            } else if (values.max < values.size) {
                message = `Max File Size is smaller that files's original size: ${values.size} bytes`;
            } else if (values.userFiles && values.userFiles.find && values.userFiles.find(createAbsPath(values.folder, values.filename))) {
                message = `File with the same name already exists in this folder`;
            }
            return {
                err: message !== undefined,
                message: message || ''
            }
        }
    );
});

var newFileDialogFileContents;

function showError(e) {
    const statusBar = document.querySelector('#statusBar');
    statusBar.showToastMessage(e.message || e.toString(), 5000);
}

function createAbsPath(folder, name) {
    let path = '/' + (folder || '');
    path = path.split('//').join('/');
    if (name) {
        path = path.length > 1 ? path + '/' + name : name;
    }
    return path.split('//').join('/').toLowerCase();
}

document.addEventListener('gc-nav-ready', function () {

    gc.nav.registerAction('view.device.status', function () {
        gc.databind.registry.getBinding('widget.vtab.selectedIndex').setValue(0);
    });

    gc.nav.registerAction('view.user.files', function () {
        gc.databind.registry.getBinding('widget.vtab.selectedIndex').setValue(1);
    });

    gc.nav.registerAction('file.add', function () {
        const dialog = document.querySelector('#fileLoadDialog');
        gc.File.browseAndLoad(null, { bin: true }, (contents) => {

            newFileDialogFileContents = contents;

            // set file size and max allocated size to match the file contents.
            const size = contents.length || contents.byteLength;
            gc.databind.registry.getBinding('prop.dialog_file_size').setValue(size);
            gc.databind.registry.getBinding('widget.dialog_file_size.label').setValue('(actual size: '+ size + ' bytes)');
            gc.databind.registry.getBinding('widget.dialog_max_size.value').setValue(size);
            gc.databind.registry.getBinding('widget.dialog_file_path.value').setValue();

            // update file name if not already specified
            gc.databind.registry.getBinding('widget.dialog_file_name.value').setValue(dialog.files[0].name.toLowerCase());

            // update file path based on filetree selection

            const activeFile = gc.databind.registry.getBinding('widget.filetree.selectedFile').getValue();
            let filepath = createAbsPath('/', gc.databind.registry.getBinding('widget.filetree.selectedFolder').getValue());
            if (activeFile && activeFile.isDirectory) {
                filepath = createAbsPath(filepath, activeFile.name);
            }
            gc.databind.registry.getBinding('widget.dialog_file_path.value').setValue(filepath || '/');

            document.querySelector('#newFileDialog').open();

        }, dialog);
    });

    gc.nav.registerAction('file.open', function () {
        const codec = document.querySelector('#uart').getCodec();
        const selectedFilePath = gc.databind.registry.getBinding('prop.activeFilePath').getValue();
        if (selectedFilePath) {
            const editorTabs = document.querySelector('#editorTabs');
            const editorTab = editorTabs.findTextEditorTab(selectedFilePath);
            const filename = gc.databind.registry.getBinding('widget.filetree.selectedFile').getValue().name;
            if (editorTab) {
                // activate existing edotor tab
                editorTabs.addTextEditorTab(filename, selectedFilePath);
            } else {
                // read file contents and create new editor tab
                codec.readFileContents(selectedFilePath).then((contents) => {
                    const { text, encoding } = codec.convertDataToText(contents);
                    editorTabs.addTextEditorTab(filename, selectedFilePath, text, encoding);
                }).catch(showError);
            }
        }
    });

    gc.nav.registerAction('file.save', async function () {
        const editorTabs = document.querySelector('#editorTabs');
        const tab = editorTabs.activeTab;
        if (tab && tab.dirty) {
            const { text, encoding } = editorTabs.getEditorText(tab.title);
            const codec = document.querySelector('#uart').getCodec();
            const data = codec.convertTextToData(text, encoding);
            try {
                await codec.writeFileContents(tab.title, data);
                editorTabs.setDirty(tab, false);
            } catch (e) {
                showError(e);
            }
        }
    });

    gc.nav.registerAction('file.delete', async function () {
        const codec = document.querySelector('#uart').getCodec();
        const selectedFilePath = gc.databind.registry.getBinding('prop.activeFilePath').getValue();
        if (selectedFilePath) {
            try {
                await codec.deleteFileContents(selectedFilePath);

                const editorTabs = document.querySelector('#editorTabs');
                editorTabs.closeTab(selectedFilePath);

                codec.refresh();
            } catch(e) {
                showError(e);
            }
        }
    });

    gc.nav.registerAction('file.upload', function () {
        const selectedFilePath = gc.databind.registry.getBinding('prop.activeFilePath').getValue();
        if (selectedFilePath) {
            gc.File.browseAndLoad(null, { bin: true }, async (contents) => {
                const codec = document.querySelector('#uart').getCodec();
                try {
                    await codec.writeFileContents(selectedFilePath, contents);

                    const editorTabs = document.querySelector('#editorTabs');
                    if (editorTabs.findTextEditorTab(selectedFilePath)) {
                        const { text, encoding } = codec.convertDataToText(new Uint8Array(contents));
                        editorTabs.setEditorText(selectedFilePath, text, encoding);
                    }
                } catch (e) {
                    showError(e);
                }

            }, document.querySelector('#fileLoadDialog'));
        }
    });

    gc.nav.registerAction('file.download', async () => {
        const selectedFilePath = gc.databind.registry.getBinding('prop.activeFilePath').getValue();
        if (selectedFilePath) {
            try {
                const editorTabs = document.querySelector('#editorTabs');
                const { text, encoding } = editorTabs.getEditorText(selectedFilePath);
                const codec = document.querySelector('#uart').getCodec();
                const data = text ? codec.convertTextToData(text, encoding) : await codec.readFileContents(selectedFilePath);
                const filename = gc.databind.registry.getBinding('widget.filetree.selectedFile').getValue().name;
                if (typeof process != 'undefined') {
                    gc.File.browseAndSave(data, filename, null, function () {
                        console.log('finished saving file');
                    }, document.querySelector('#fileSaveDialog'));
                }
                else {
                    gc.File.saveBrowserFile(data, {
                        filename: filename,
                        bin: true
                    }, function () {
                        console.log('finished saving file');
                    });
                }

            } catch (e) {
                showError(e);
            }
        }
    });

    gc.nav.registerAction('view.refresh', function () {
        var codec = document.querySelector('#uart').getCodec();
        codec.refresh().catch(showError);
    });
    gc.nav.disableAction('view.refresh');  // start disabled, and wait for h/w to connect.

});

document.addEventListener('DOMContentLoaded', function () {
    let firstTime = true;
    const templateObj = document.querySelector('#template_obj');
    templateObj.isCCS = gc.desktop.isCCS();

    templateObj.addEventListener('dom-change', function () {
        if (firstTime) {
            const fileTree = templateObj.$.filetree;
            fileTree.addEventListener('onOpenFile', function () {
                gc.nav.onClick('file.open');
            });
            document.querySelector('#editorTabs').addOtherTab('Properties', 'File and Folder Properties', undefined, true);

            // keep file tree selection in sync with editor tab.
            const editorTabs = document.querySelector('#editorTabs');
            editorTabs.addEventListener('active-tab-changed', () => {
                if (editorTabs.isTextEditorTabActive) {
                    const filePath = editorTabs.activeTabTitle;
                    const index = fileTree.nodes.findIndex(node => node.fileObj.path === filePath);
                    if (index >= 0) {
                        fileTree.selectedIndex = index;
                    }
                }
            });
            editorTabs.addEventListener('onDirtyChanged', (event) => {
                const tab = event.detail;
                if (tab && tab.dirty) {
                    gc.nav.enableAction('file.save');
                } else {
                    gc.nav.disableAction('file.save');
                }
            });

            document.querySelector('#dialog_submit').addEventListener('click', async function(e) {
                try {
                    const codec = document.querySelector('#uart').getCodec();
                    const filename = gc.databind.registry.getBinding('widget.dialog_file_name.value').getValue();
                    const selectedFilePath = createAbsPath(gc.databind.registry.getBinding('widget.dialog_file_path.value').getValue(), filename)
                    const maxSize = gc.databind.registry.getBinding('widget.dialog_max_size.value').getValue();
                    const failsafe = gc.databind.registry.getBinding('widget.dialog_failsafe.checked').getValue();

                    await codec.writeFileContents(selectedFilePath, newFileDialogFileContents || '', {
                        flagFailSafe: failsafe,
                        maxsize: maxSize
                    });

                    const editorTabs = document.querySelector('#editorTabs');
                    if (editorTabs.findTextEditorTab(selectedFilePath)) {
                        const { text, encoding } = codec.convertDataToText(new Uint8Array(newFileDialogFileContents || ''));
                        editorTabs.setEditorText(selectedFilePath, text, encoding);
                    }

                    await codec.refresh();
                } catch (e) {
                    showError(e);
                }
            });

            // work-around for CCS not calling updateStyles when changing from ti-theme to a CCS theme.
            setTimeout(() => {
                Polymer.updateStyles();
            }, 1000)
        }
        firstTime = false;
    });

    gc.autoDetectPortsFilter = function (transport, ports, info) {

        info.setRecommendedPort(ports.reduce((result, port) => {
            if (port.productId && port.productId.toLowerCase() === 'bef3' && 
                    port.vendorId && port.vendorId.toLowerCase() === '0451'&& port.pnpId.endsWith('00')) {
                return port;
            }
            return result;
        }, undefined));
    };
});
